@@ -1,4 +1,4 @@
-"""Ansible runtime environment maanger."""
+"""Ansible runtime environment manager."""
 import importlib
 import json
 import logging
@@ -46,6 +46,9 @@ class Runtime:
 
     _version: Optional[packaging.version.Version] = None
     cache_dir: Optional[str] = None
+    # Used to track if we have already initialized the Ansible runtime as attempts
+    # to do it multiple tilmes will cause runtime warnings from within ansible-core
+    initialized: bool = False
 
     # pylint: disable=too-many-arguments
     def __init__(
@@ -68,11 +71,11 @@ class Runtime:
         :param min_required_version: Minimal version of Ansible required. If
                                      not found, a :class:`RuntimeError`
                                      exception is raised.
-        :param: require_module: If set, instantiation will fail if Ansible
-                                Python module is missing or is not matching
-                                the same version as the Ansible command line.
-                                That is useful for consumers that expect to
-                                also perform Python imports from Ansible.
+        :param require_module: If set, instantiation will fail if Ansible
+                               Python module is missing or is not matching
+                               the same version as the Ansible command line.
+                               That is useful for consumers that expect to
+                               also perform Python imports from Ansible.
         :param max_retries: Number of times it should retry network operations.
                             Default is 0, no retries.
         :param environ: Environment dictionary to use, if undefined
@@ -88,7 +91,7 @@ class Runtime:
         # As CryptographyDeprecationWarning is not a builtin, we cannot use
         # PYTHONWARNINGS to ignore it using category but we can use message.
         # https://stackoverflow.com/q/68251969/99834
-        if "PYTHONWARNINGS" not in self.environ:
+        if "PYTHONWARNINGS" not in self.environ:  # pragma: no cover
             self.environ["PYTHONWARNINGS"] = "ignore:Blowfish has been deprecated"
 
         if isolated:
@@ -123,6 +126,32 @@ class Runtime:
                 "indicates a broken execution environment."
             )
 
+        # For ansible 2.15+ we need to initialize the plugin loader
+        # https://github.com/ansible/ansible-lint/issues/2945
+        if not Runtime.initialized:  # noqa: F823
+            col_path = [f"{self.cache_dir}/collections"]
+            if self.version >= Version("2.15.0.dev0"):
+                # pylint: disable=import-outside-toplevel,no-name-in-module
+                from ansible.plugins.loader import init_plugin_loader
+
+                init_plugin_loader(col_path)
+            else:
+                # noinspection PyProtectedMember
+                from ansible.utils.collection_loader._collection_finder import (  # pylint: disable=import-outside-toplevel
+                    _AnsibleCollectionFinder,
+                )
+
+                # noinspection PyProtectedMember
+                # pylint: disable=protected-access
+                col_path += self.config.collections_paths
+                col_path += os.path.dirname(
+                    os.environ.get(ansible_collections_path(), ".")
+                ).split(":")
+                _AnsibleCollectionFinder(
+                    paths=col_path
+                )._install()  # pylint: disable=protected-access
+            Runtime.initialized = True
+
     def clean(self) -> None:
         """Remove content of cache_dir."""
         if self.cache_dir:
@@ -214,10 +243,7 @@ class Runtime:
             "install",
             "-vvv",  # this is needed to make ansible display important info in case of failures
         ]
-
-        # ansible-galaxy before 2.11 fails to upgrade collection unless --force
-        # is present, newer versions do not need it
-        if force or self.version_in_range(upper="2.11"):
+        if force:
             cmd.append("--force")
 
         # As ansible-galaxy install is not able to automatically determine
@@ -227,14 +253,19 @@ class Runtime:
         if matches and Version(matches[1]).is_prerelease:
             cmd.append("--pre")
 
-        if destination:
-            cmd.extend(["-p", str(destination)])
+        cpaths: List[str] = self.config.collections_paths
+        if destination and str(destination) not in cpaths:
+            # we cannot use '-p' because it breaks galaxy ability to ignore already installed collections, so
+            # we hack ansible_collections_path instead and inject our own path there.
+            # pylint: disable=no-member
+            cpaths.insert(0, str(destination))
         cmd.append(f"{collection}")
 
         _logger.info("Running from %s : %s", os.getcwd(), " ".join(cmd))
         run = self.exec(
             cmd,
             retry=True,
+            env={**self.environ, ansible_collections_path(): ":".join(cpaths)},
         )
         if run.returncode != 0:
             msg = f"Command returned {run.returncode} code:\n{run.stdout}\n{run.stderr}"
@@ -270,10 +301,16 @@ class Runtime:
                     force=True,
                 )
 
+    # pylint: disable=too-many-branches
     def install_requirements(
         self, requirement: str, retry: bool = False, offline: bool = False
     ) -> None:
-        """Install dependencies from a requirements.yml."""
+        """Install dependencies from a requirements.yml.
+
+        :param requirement: path to requirements.yml file
+        :param retry: retry network operations on failures
+        :param offline: bypass installation, may fail if requirements are not met.
+        """
         if not os.path.exists(requirement):
             return
         reqs_yaml = yaml_from_file(requirement)
@@ -295,17 +332,18 @@ class Runtime:
 
             if offline:
                 _logger.warning(
-                    "Offline mode ignored because `ansible-galaxy role install` command does not support it."
+                    "Skipped installing old role dependencies due to running in offline mode."
                 )
-            _logger.info("Running %s", " ".join(cmd))
-            result = self.exec(cmd, retry=retry)
-            if result.returncode != 0:
-                _logger.error(result.stdout)
-                raise AnsibleCommandError(result)
+            else:
+                _logger.info("Running %s", " ".join(cmd))
+
+                result = self.exec(cmd, retry=retry)
+                if result.returncode != 0:
+                    _logger.error(result.stdout)
+                    raise AnsibleCommandError(result)
 
         # Run galaxy collection install works on v2 requirements.yml
         if "collections" in reqs_yaml:
-
             cmd = [
                 "ansible-galaxy",
                 "collection",
@@ -313,21 +351,29 @@ class Runtime:
                 "-v",
             ]
             if offline:
-                if self.version_in_range(upper="2.14"):
-                    _logger.warning(
-                        "Offline mode ignored because it is not supported by ansible versions before 2.14."
-                    )
-                else:
-                    cmd.append("--offline")
-            cmd.extend(["-r", requirement])
-            if self.cache_dir:
-                cmd.extend(["-p", f"{self.cache_dir}/collections"])
-            _logger.info("Running %s", " ".join(cmd))
-            result = self.exec(cmd, retry=retry)
-            if result.returncode != 0:
-                _logger.error(result.stdout)
-                _logger.error(result.stderr)
-                raise AnsibleCommandError(result)
+                _logger.warning(
+                    "Skipped installing collection dependencies due to running in offline mode."
+                )
+            else:
+                cmd.extend(["-r", requirement])
+                cpaths = self.config.collections_paths
+                if self.cache_dir:
+                    # we cannot use '-p' because it breaks galaxy ability to ignore already installed collections, so
+                    # we hack ansible_collections_path instead and inject our own path there.
+                    dest_path = f"{self.cache_dir}/collections"
+                    if dest_path not in cpaths:
+                        # pylint: disable=no-member
+                        cpaths.insert(0, dest_path)
+                _logger.info("Running %s", " ".join(cmd))
+                result = self.exec(
+                    cmd,
+                    retry=retry,
+                    env={**os.environ, "ANSIBLE_COLLECTIONS_PATH": ":".join(cpaths)},
+                )
+                if result.returncode != 0:
+                    _logger.error(result.stdout)
+                    _logger.error(result.stderr)
+                    raise AnsibleCommandError(result)
 
     def prepare_environment(  # noqa: C901
         self,
